"""
This file is automatically generated by TritonParse reproducer.
It contains a smallest testing example for a Triton kernel.

{{REPRODUCER_METADATA_PLACEHOLDER}}
"""

import logging

import torch

logger = logging.getLogger(__name__)

# {{CONTEXT_JSON_PLACEHOLDER}}

# {{COMPILATION_JSON_PLACEHOLDER}}

# {{IR_OVERRIDE_SETUP_PLACEHOLDER}}

# {{UTILITY_FUNCTIONS_PLACEHOLDER}}

# isort: off
# {{KERNEL_SYSPATH_PLACEHOLDER}}

# {{KERNEL_IMPORT_PLACEHOLDER}}
# isort: on


def launch_kernel():
    # {{LAUNCH_KERNEL_BODY_PLACEHOLDER}}

    print("Generated kernel arguments dictionary:")
    for name, arg in args_dict.items():  # noqa: F821
        print(f"  {name}: {arg}")
    print(f"Grid: {grid}")  # noqa: F821

    # {{KERNEL_INVOCATION_PLACEHOLDER}}

    torch.cuda.synchronize()
    print("Kernel execution finished.")


if __name__ == "__main__":
    launch_kernel()
