// Custom icons (not available in Heroicons)
export { GitHubIcon } from "./GitHubIcon";

// Re-export Heroicons for convenient access
// Outline icons (24x24, stroke-based)
export {
  ArrowLeftIcon,
  ArrowsRightLeftIcon,
  BookOpenIcon,
  CheckIcon,
  ChevronRightIcon,
  ClipboardDocumentIcon,
  DocumentDuplicateIcon,
  DocumentPlusIcon,
  DocumentTextIcon,
  LinkIcon,
  PaperClipIcon,
  ShareIcon,
} from "@heroicons/react/24/outline";

// Solid icons (20x20, fill-based) - used for some filled variants
export {
  ChevronRightIcon as ChevronRightSolidIcon,
  ShareIcon as ShareSolidIcon,
} from "@heroicons/react/20/solid";
